import os
import importlib

from .config import *
from .manual import *

__all__ = "Plugin",

class Plugin():
    """Store the plugin and its location information

    Class Properties:
    ```
        inserted_plugins:dict
    ```

    Instance Properties:
    ```
        plugin_list:list | None
        search_path:str | None
    ```
    """
    inserted_plugins:dict = {}
    """
    ```
    {
        module:'module' : path:str,
        ...
    }
    ```
    """
    def __init__(self, plugin_list = None, search_path = None):
        self.plugin_list = plugin_list
        self.search_path = search_path


    def _path_name(self, name:str = '', path:str = '') -> str:
        if path == None:
            return name
        elif path == self.search_path:
            i_name = Config.path_plugins.split(Config.path_pybot)[-1].lstrip("/")
            name =  ".".join(i_name.split("/")) + f".{name}"
            return name
        elif path == Config.path_plugins:
            i_name = Config.path_plugins.split(Config.path_pybot)[-1].lstrip("/")
            name =  ".".join(i_name.split("/")) + f".{name}"
            return name


    def insert_plugin_(self, name:str):
        _plugin_search_list = os.listdir(self.search_path)
        """Default launcher path"""
        plugin_search_list = [i.rstrip(".py") for i in _plugin_search_list]

        if os.path.exists(Config.path_plugins):
            _plugin_src_list = os.listdir(Config.path_plugins)
            plugin_src_list = [i.rstrip(".py") for i in _plugin_src_list]
        else:
            plugin_src_list = []

        if name in plugin_search_list:
            module = importlib.import_module(
                self._path_name(name, self.search_path)
            )
            if self.search_path != None:
                self.inserted_plugins.update(
                    {
                        module:self.search_path + f"/{name}"
                    }
                )
            else:
                self.inserted_plugins.update(
                    {
                        module:Config.path_pybot + f"/{name}"
                    }
                )
        elif name in plugin_src_list:
            module = importlib.import_module(
                self._path_name(name, Config.path_plugins)
            )
            self.inserted_plugins.update(
                {
                    module:Config.path_plugins + f"/{name}"
                }
            )
        else:
            raise RuntimeError(f"未找到{name}!!!")


    def insert_plugins_(self, dir_path:str):
        dir_path_abs = Config.path_pybot + "/" + dir_path.replace(".","/")
        dir_list = os.listdir(dir_path_abs)
        for _ in dir_list:
            name = dir_path + f".{_}"
            module = importlib.import_module(name)
            self.inserted_plugins.update(
                {
                    module:dir_path_abs + "/" + _
                }
            )


    @classmethod
    def look_inserted_plugins(cls):
        return cls.inserted_plugins
